/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.isobus;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Arrays;
import java.util.List;
import java.util.logging.Level;
import nl.cloudfarming.client.isobus.model.Isobus;
import nl.cloudfarming.client.logging.AppLogFactory;
import nl.cloudfarming.client.logging.AppLogger;
import nl.cloudfarming.client.util.FilePickerUtil;
import org.openide.awt.ActionID;
import org.openide.awt.ActionReference;
import org.openide.awt.ActionRegistration;
import org.openide.filesystems.FileChooserBuilder;
import org.openide.util.Exceptions;
import org.openide.util.NbBundle;

/**
 *
 * @author Gerben Feenstra
 */
@ActionID(id = "nl.cloudfarming.client.isobus.IsobusFilePicker", category = "Tools")
@ActionRegistration(iconInMenu = true, displayName = "#isobus.filepicker.menu.displayname", iconBase = "nl/cloudfarming/client/isobus/iso_add.png")
@ActionReference(path = "Menu/Tools", position = 60)
public class IsobusFilePicker implements ActionListener {

    private static final AppLogger LOG = AppLogFactory.getLogger(IsobusFilePicker.class);
    private static final String EXTENSION = NbBundle.getMessage(IsobusModule.class, "isobus.filepicker.filefilter.extension");
    private static final String EXTENSION_DESCRIPTION = NbBundle.getMessage(IsobusModule.class, "isobus.filepicker.filefilter.description", EXTENSION);
    private static final String FILEPICKER_TITLE = NbBundle.getMessage(IsobusModule.class, "isobus.filepicker.title");
    private static final String FILEPICKER_APPROVE = NbBundle.getMessage(IsobusModule.class, "isobus.filepicker.approve");
    private File selectedFile = null;

    public IsobusFilePicker() {
    }

    /**
     *
     * @param e
     */
    @Override
    public void actionPerformed(ActionEvent e) {
        // Retreive last workingdirectory
        File workingDirectory = new File(IsobusPreference.LAST_DIR.getValue());

        // Build the filechooser
        FileChooserBuilder fcb = new FileChooserBuilder("Isobus-directory");
        fcb.setTitle(FILEPICKER_TITLE);
        fcb.setDefaultWorkingDirectory(workingDirectory);
        fcb.setApproveText(FILEPICKER_APPROVE);

        List<String> xmlExtensions = Arrays.asList(new String[]{EXTENSION});
        fcb.addFileFilter(FilePickerUtil.createSwingFileFilter(xmlExtensions, EXTENSION_DESCRIPTION));

        // Open dialog, retreive selected sensorfile
        selectedFile = fcb.showOpenDialog();

        if (selectedFile != null && selectedFile.isFile()) {

            // start processing of files

            //Thread processThread = new Thread(new ProcessJob());
            //processThread.start();

            // Not running in thread, because reading XML is probably hardly noticable.
            new ProcessJob().run();

        }

    }

    class ProcessJob implements Runnable {

        @Override
        public void run() {
            IsobusMarshaller instance = new IsobusMarshaller();


            BufferedReader br;
            StringBuilder xml = new StringBuilder();
            try {
                br = new BufferedReader(new FileReader(selectedFile));
                String line = "";
                while ((line = br.readLine()) != null) {
                    xml.append(line);
                }
            } catch (FileNotFoundException ex) {
                Exceptions.printStackTrace(ex);
            } catch (IOException ex) {
                Exceptions.printStackTrace(ex);
            }

            Isobus result = instance.unMarshall(xml.toString());

            if (result == null) {
                LOG.log(Level.INFO, "File {0} did not contain isoxml data", selectedFile.getName());
            } else {
                // Display info in log
                LOG.log(Level.INFO, "File {0} has been processed", selectedFile.getName());
                LOG.info("Imported The following data:");
                if (result.getCCG().size() > 0) {
                    LOG.log(Level.INFO, "{0} x {1}", new Object[]{result.getCCG().size(), result.getCCG().get(0).getClass().getName().replace("nl.cloudfarming.client.isobus.model.", "")});
                }
                if (result.getCCT().size() > 0) {
                    LOG.log(Level.INFO, "{0} x {1}", new Object[]{result.getCCT().size(), result.getCCT().get(0).getClass().getName().replace("nl.cloudfarming.client.isobus.model.", "")});
                }
                if (result.getCLD().size() > 0) {
                    LOG.log(Level.INFO, "{0} x {1}", new Object[]{result.getCLD().size(), result.getCLD().get(0).getClass().getName().replace("nl.cloudfarming.client.isobus.model.", "")});
                }
                if (result.getCTP().size() > 0) {
                    LOG.log(Level.INFO, "{0} x {1}", new Object[]{result.getCTP().size(), result.getCTP().get(0).getClass().getName().replace("nl.cloudfarming.client.isobus.model.", "")});
                }
                if (result.getCTR().size() > 0) {
                    LOG.log(Level.INFO, "{0} x {1}", new Object[]{result.getCTR().size(), result.getCTR().get(0).getClass().getName().replace("nl.cloudfarming.client.isobus.model.", "")});
                }
                if (result.getDVC().size() > 0) {
                    LOG.log(Level.INFO, "{0} x {1}", new Object[]{result.getDVC().size(), result.getDVC().get(0).getClass().getName().replace("nl.cloudfarming.client.isobus.model.", "")});
                }
                if (result.getFRM().size() > 0) {
                    LOG.log(Level.INFO, "{0} x {1}", new Object[]{result.getFRM().size(), result.getFRM().get(0).getClass().getName().replace("nl.cloudfarming.client.isobus.model.", "")});
                }
                if (result.getOTQ().size() > 0) {
                    LOG.log(Level.INFO, "{0} x {1}", new Object[]{result.getOTQ().size(), result.getOTQ().get(0).getClass().getName().replace("nl.cloudfarming.client.isobus.model.", "")});
                }
                if (result.getPDT().size() > 0) {
                    LOG.log(Level.INFO, "{0} x {1}", new Object[]{result.getPDT().size(), result.getPDT().get(0).getClass().getName().replace("nl.cloudfarming.client.isobus.model.", "")});
                }
                if (result.getPFD().size() > 0) {
                    LOG.log(Level.INFO, "{0} x {1}", new Object[]{result.getPFD().size(), result.getPFD().get(0).getClass().getName().replace("nl.cloudfarming.client.isobus.model.", "")});
                }
                if (result.getPGP().size() > 0) {
                    LOG.log(Level.INFO, "{0} x {1}", new Object[]{result.getPGP().size(), result.getPGP().get(0).getClass().getName().replace("nl.cloudfarming.client.isobus.model.", "")});
                }
                if (result.getTSK().size() > 0) {
                    LOG.log(Level.INFO, "{0} x {1}", new Object[]{result.getTSK().size(), result.getTSK().get(0).getClass().getName().replace("nl.cloudfarming.client.isobus.model.", "")});
                }
                if (result.getVPN().size() > 0) {
                    LOG.log(Level.INFO, "{0} x {1}", new Object[]{result.getVPN().size(), result.getVPN().get(0).getClass().getName().replace("nl.cloudfarming.client.isobus.model.", "")});
                }
                if (result.getWKR().size() > 0) {
                    LOG.log(Level.INFO, "{0} x {1}", new Object[]{result.getWKR().size(), result.getWKR().get(0).getClass().getName().replace("nl.cloudfarming.client.isobus.model.", "")});
                }
                if (result.getXFR().size() > 0) {
                    LOG.log(Level.INFO, "{0} x {1}", new Object[]{result.getXFR().size(), result.getXFR().get(0).getClass().getName().replace("nl.cloudfarming.client.isobus.model.", "")});
                }
            }

        }
    }
}
